//notation: js file can only use this kind of comments
//since comments will cause error when use in webview.loadurl,
//comments will be remove by java use regexp
(function() {
    if (window.WebViewJavascriptBridge) {
        return;
    }

    var messagingIframe;
    var sendMessageQueue = [];
    var receiveMessageQueue = [];
    var messageHandlers = {};

    var msgQueueIframeHandlers = {};

    var msgQueueIframeId = [];

    var lastCallTime = 0;

    var maxIframeCount = 10;

    var stoId = null;

    var CUSTOM_PROTOCOL_SCHEME = 'wvjbscheme';
    var QUEUE_HAS_MESSAGE = '__WVJB_QUEUE_MESSAGE__';

    var responseCallbacks = {};
    var uniqueId = 1;

    // 创建队列iframe
    function _createQueueReadyIframe(doc) {
        messagingIframe = doc.createElement('iframe');
        messagingIframe.style.display = 'none';
        doc.documentElement.appendChild(messagingIframe);
    }

    //set default messageHandler  初始化默认的消息线程
    function init(messageHandler) {
        if (WebViewJavascriptBridge._messageHandler) {
            throw new Error('WebViewJavascriptBridge.init called twice');
        }
        WebViewJavascriptBridge._messageHandler = messageHandler;
        var receivedMessages = receiveMessageQueue;
        receiveMessageQueue = null;
        for (var i = 0; i < receivedMessages.length; i++) {
            _dispatchMessageFromNative(receivedMessages[i]);
        }
    }

    // 发送
    function send(data, responseCallback) {
        _doSend({
            data: data
        }, responseCallback);
    }

    // 注册线程 往数组里面添加值
    function registerHandler(handlerName, handler) {
        messageHandlers[handlerName] = handler;
    }
    // 调用线程
    function callHandler(handlerName, data, responseCallback) {
        _doSend({
            handlerName: handlerName,
            data: data
        }, responseCallback);
    }

    //sendMessage add message, 触发native处理 sendMessage
    function _doSend(message, responseCallback) {

        if (responseCallback) {
            var callbackId = 'cb_' + (uniqueId++) + '_' + new Date().getTime();
            responseCallbacks[callbackId] = responseCallback;
            message.callbackId = callbackId;
        }
        sendMessageQueue.push(message);

        messagingIframe.src = CUSTOM_PROTOCOL_SCHEME + '://' + QUEUE_HAS_MESSAGE;

    }

    // 提供给native调用,该函数作用:获取sendMessageQueue返回给native,由于android不能直接获取返回的内容,所以使用url shouldOverrideUrlLoading 的方式返回内容
    function _fetchQueue() {

        // 空数组直接返回
        if (sendMessageQueue.length === 0) {
          return;
        }


        var msgQueueLen = sendMessageQueue.length;

        var msgQueueIframeTag = 'iframe_'+ new Date().getTime();

        for(var i=0;i<msgQueueLen;i++){
            var sendMsgItem = sendMessageQueue[i];
            msgQueueIframeHandlers[sendMsgItem.callbackId] = msgQueueIframeTag;
        }

        var messageQueueString = JSON.stringify(sendMessageQueue);
        sendMessageQueue = [];

        var iframe = document.createElement('iframe');
        iframe.id = ""+msgQueueIframeTag;
        iframe.style = "display:none;";
        iframe.src= CUSTOM_PROTOCOL_SCHEME + '://return/_fetchQueue/' + encodeURIComponent(messageQueueString);
        doc.documentElement.appendChild(iframe);

        msgQueueIframeId.push(iframe.id);

         var iframesTest1 = doc.getElementsByTagName("iframe");

         if(iframesTest1.length > maxIframeCount){
            var iframeId = msgQueueIframeId.shift();
            var iframe = doc.getElementById(iframeId);
            iframe.parentNode.removeChild(iframe);
         }
    }

    //提供给native使用,
    function _dispatchMessageFromNative(messageJSON) {
        setTimeout(function() {
            var message = JSON.parse(messageJSON);
            var responseCallback;
            //java call finished, now need to call js callback function
            if (message.responseId) {
                responseCallback = responseCallbacks[message.responseId];
                if (!responseCallback) {
                    return;
                }
                responseCallback(message.responseData);
                delete responseCallbacks[message.responseId];
                //移除iframe
                destroyIframe(message.responseId);
            } else {
                //直接发送
                if (message.callbackId) {
                    var callbackResponseId = message.callbackId;
                    responseCallback = function(responseData) {
                        _doSend({
                            responseId: callbackResponseId,
                            responseData: responseData
                        });
                    };
                }

                var handler = WebViewJavascriptBridge._messageHandler;
                if (message.handlerName) {
                    handler = messageHandlers[message.handlerName];
                }
                //查找指定handler
                try {
                    //console.log("dispatchMessageFromNative2->"+message.data);
                    handler(message.data, responseCallback);
                } catch (exception) {
                    console.log("exception->"+exception);
                    if (typeof console != 'undefined') {
                        console.log("WebViewJavascriptBridge: WARNING: javascript handler threw.", message, exception);
                    }
                }
            }
        });
    }


    /**
    * 销毁iframe，释放iframe所占用的内存。
    * @param iframe 需要销毁的iframe id
    */
    function destroyIframe(responseId){
            var iframeId = msgQueueIframeHandlers[responseId];
            var iframe = doc.getElementById(iframeId);
            iframe.parentNode.removeChild(iframe);
    }



    //提供给native调用,receiveMessageQueue 在会在页面加载完后赋值为null,所以
    function _handleMessageFromNative(messageJSON) {
        if (receiveMessageQueue) {
            receiveMessageQueue.push(messageJSON);
        }
        _dispatchMessageFromNative(messageJSON);

    }

    var WebViewJavascriptBridge = window.WebViewJavascriptBridge = {

        init: init,
        send: send,
        registerHandler: registerHandler,
        callHandler: callHandler,
        _fetchQueue: _fetchQueue,
        _handleMessageFromNative: _handleMessageFromNative
    };

    var doc = document;
    _createQueueReadyIframe(doc);
//    var readyEvent = doc.createEvent('Events');
//    readyEvent.initEvent('WebViewJavascriptBridgeReady');
//    readyEvent.bridge = WebViewJavascriptBridge;
//    doc.dispatchEvent(readyEvent);

    setTimeout(_callWVJBCallbacks, 0);
    function _callWVJBCallbacks() {
        var callbacks = window.WVJBCallbacks;
        delete window.WVJBCallbacks;
        for (var i=0; i<callbacks.length; i++) {
            callbacks[i](WebViewJavascriptBridge);
        }
    }
})();